#!/usr/bin/python

from mod_utils              import *
from mod_ecu_screen         import *
from mod_ecu_default        import *
from mod_ecu_state          import *
from mod_ecu_parameter      import *
from mod_ecu_command        import *
from mod_ecu_identification import *
from mod_ecu_service        import *
from mod_ecu_mnemonic       import *
from mod_optfile            import *
from mod_ply                import *
from mod_elm                import ELM
from mod_elm                import snat
from mod_elm                import dnat

import mod_globals

from xml.dom.minidom        import parse
from datetime               import datetime
import xml.dom.minidom
import xml.etree.ElementTree as et
import struct
import sys
import os
import time

#sys.setdefaultencoding('utf-8')

F2A = {"01":"7A", "02":"01", "03":"51", "04":"26", "05":"2C", "06":"00", "07":"24", "08":"29", "09":"6E",  
		 "10":"57", "11":"52", "12":"79", "13":"0D", "14":"00", "15":"32", "16":"37", "17":"6B",  
		 "18":"04", "19":"3F", "20":"27", "21":"08", "22":"00", "23":"3A", "24":"50", "25":"1C",  
		 "26":"00", "27":"99", "28":"00", "29":"07", "30":"66", "31":"A7", "32":"60", "33":"4B",  
		 "34":"2B", "35":"1B", "36":"61", "38":"1E", "40":"23", "41":"0E", "42":"40", "43":"7C",  
		 "47":"4D", "50":"02", "51":"0F", "52":"70", "53":"71", "54":"72", "55":"0E", "56":"1A",  
		 "60":"A5", "61":"A6", "62":"00", "63":"65", "65":"2A", "66":"FE", "70":"62", "72":"00",  
		 "73":"63", "76":"13", "77":"77", "78":"64", "79":"D1", "80":"F7", "81":"F8", "86":"2E",  
		 "87":"06", "90":"59", "91":"86", "92":"87", "93":"00", "94":"67", "95":"93", "96":"95",  
		 "97":"68", "98":"A8", "99":"C0"} 
class ECU:
  '''Contains data for one specific ECU
     implement menu for ecu'''
  
  path = "../EcuRenault/Sessions/"
 
  getDTCmnemo     = ""
  resetDTCcommand = ""
  screens         = []
  Defaults        = {}
  Parameters      = {}
  States          = {}
  Identifications = {}
  Commands        = {}
  Services        = {}
  Mnemonics       = {}
  
  ecudata  = {}
  
  minimumrefreshrate = 0.200
    
  def __init__(self, cecu, tran ):
    
    self.elm     = 0
    self.ecudata = cecu

    self.getDTCmnemo     = ""
    self.resetDTCcommand = ""
    self.screens         = []
    self.Defaults        = {}
    self.Parameters      = {}
    self.States          = {}
    self.Identifications = {}
    self.Commands        = {}
    self.Services        = {}
    self.Mnemonics       = {}
    
    print 'Deflen:',len(self.Defaults)

    print "ECU type: ", cecu['stdType']
 
    mdom = xml.dom.minidom.parse(self.path+self.ecudata['ModelId'])
    mdoc = mdom.documentElement
  
    print "Loading screens "
    self.screens = []
    sc_class = ecu_screens( self.screens, mdoc, tran )

    print "Loading optimyzer"
    self.defaults = []
    opt_file = optfile(self.path+self.ecudata['OptimizerId'])
       
    print "Loading defaults"
    df_class = ecu_defaults       ( self.Defaults,        mdoc, opt_file.dict, tran )
    print "Loading parameters"
    pr_class = ecu_parameters     ( self.Parameters,      mdoc, opt_file.dict, tran )
    print "Loading states"
    st_class = ecu_states         ( self.States,          mdoc, opt_file.dict, tran )
    print "Loading identifications"
    id_class = ecu_identifications( self.Identifications, mdoc, opt_file.dict, tran )
    print "Loading commands"
    cm_class = ecu_commands       ( self.Commands,        mdoc, opt_file.dict, tran )
    print "Loading services"
    sv_class = ecu_services       ( self.Services,        mdoc, opt_file.dict, tran )
    print "Loading mnemonics"
    mm_class = ecu_mnemonics      ( self.Mnemonics,       mdoc, opt_file.dict, tran )
    print "Loading DTC commands"
    self.getDTCmnemo, self.resetDTCcommand = df_class.getDTCCommands( mdoc, opt_file.dict, cecu['stdType'] )
      
  def initELM(self, elm):

    print "Loading PLY "
    self.calc = Calc()
    
    print "Init ELM"

    self.elm = elm

    if self.ecudata['pin'].lower()=='can':
      self.elm.init_can()
      self.elm.set_can_addr( self.ecudata['dst'], self.ecudata )
      self.elm.start_session_can( self.ecudata['startDiagReq'] )
    else:
      self.elm.init_iso()
      self.elm.set_iso_addr( self.ecudata['dst'], self.ecudata )
      self.elm.start_session_iso( self.ecudata['startDiagReq'] )

    print "Done"  
    
  def get_st( self, name ):
    if name not in self.States.keys():
      for i in self.States.keys():
        if name==self.States[i].codeMR:
          name = i
          break
    if name not in self.States.keys():
       return 'none','unknown state'      
    self.elm.clear_cache()
    datastr, help, csvd = get_state( self.States[name], self.Mnemonics, self.Services, self.elm, self.calc )
    return csvd, datastr
      
  def get_pr( self, name ):
    if name not in self.Parameters.keys():
      for i in self.Parameters.keys():
        if name==self.Parameters[i].codeMR:
          name = i
          break
    if name not in self.Parameters.keys():
      return 'none','unknown parameter'      
    self.elm.clear_cache()
    datastr, help, csvd = get_parameter( self.Parameters[name], self.Mnemonics, self.Services, self.elm, self.calc )
    return csvd, datastr
      
  def get_id( self, name ):
    if name not in self.Identifications.keys():
      for i in self.Identifications.keys():
        if name==self.Identifications[i].codeMR:
          name = i
          break
    if name not in self.Identifications.keys():
      return 'none','unknown identification'      
    self.elm.clear_cache()
    datastr, help, csvd = get_identification( self.Identifications[name], self.Mnemonics, self.Services, self.elm, self.calc )
    return csvd, datastr

  def get_val( self, name ):
    r1, r2 = self.get_st( name )
    if r1!='none': return r1, r2
    r1, r2 = self.get_pr( name )
    if r1!='none': return r1, r2
    r1, r2 = self.get_id( name )
    if r1!='none': return r1, r2
    return 'none','unknown name' 
    
  def run_cmd( self, name, param = '', partype = 'HEX' ):
    if name not in self.Commands.keys():
      for i in self.Commands.keys():
        if name==self.Commands[i].codeMR:
          name = i
          break
    if name not in self.Commands.keys():
      return 'none'      
    self.elm.clear_cache()
    resp = runCommand( self.Commands[name], self, self.elm, param, partype )
    return resp
      
  def show_commands(self, datarefs, path):
    while(True): 
      clearScreen()
      header =  "ECU : "+self.ecudata['ecuname']+'  '+self.ecudata['doc']+'\n'
      header = header + "Screen : "+path 
      print pyren_encode(header)   
      menu = []
      cmds = []
      for dr in datarefs:
        datastr = dr.name;
        if dr.type=='State':
          datastr = self.States[dr.name].name + 'States not supported on one screen with commands'
        if dr.type=='Parameter':
          datastr = self.Parameters[dr.name].name + 'Parameters not supported on one screen with commands'
        if dr.type=='Identification':
          datastr = self.Identifications[dr.name].name + 'Identifications not supported on one screen with commands'
        if dr.type=='Command':
          datastr = self.Commands[dr.name].codeMR + " [Command] " + self.Commands[dr.name].label
          cmds.append( dr.name )
        menu.append( datastr )

      menu.append("<Up>")
      choice = ChoiceLong(menu, "Choose :", header)
      if choice[0]=="<Up>": return
      
      header = header +" -> "+cmds[int(choice[1])-1] + " [Command] "
      executeCommand( self.Commands[cmds[int(choice[1])-1]], self, self.elm, header )
         
  def show_datarefs(self, datarefs, path):
  
    csvf = 0
  
    #Check if datarefs contains any commands
    for dr in datarefs:
      if dr.type=='Command':
        self.show_commands(datarefs, path)
        return

    if mod_globals.opt_csv:
      # prepare to csv save
      csvline = "time"
      nparams = 0
      for dr in datarefs:
        if dr.type=='State':
          csvline = csvline + ";" + self.States[dr.name].codeMR
          nparams += 1
        if dr.type=='Parameter':
          csvline = csvline + ";" + self.Parameters[dr.name].codeMR
          nparams += 1
      if nparams:
        csv_filename = datetime.now().strftime("%y_%m_%d_%H_%M_%S_")
        csv_filename = csv_filename+self.ecudata['ecuname']+'_'+path+".csv"
        csv_filename = csv_filename.replace('/','_')
        csv_filename = csv_filename.replace(' : ','_')
        csv_filename = csv_filename.replace(' -> ','_')
        csvf = open("./csv/"+csv_filename, "wt")
                
    kb = KBHit()

    tb = time.time()   #time of begining 

    if len(datarefs)==0 and 'DE' not in path: return
    
    page = 0
        
    while(True):
      
      strlst = []
      
      if mod_globals.opt_csv and csvf!=0:
        csvline = csvline + "\n"
        csvline = csvline.replace('.',',')
        csvf.write( csvline )
        csvf.flush() 
        csvline = datetime.now().strftime("%H:%M:%S.%f")
      
      self.elm.clear_cache()
      
      for dr in datarefs:
        datastr = dr.name;
        help    = dr.type
        if dr.type=='State':
          datastr, help, csvd = get_state( self.States[dr.name], self.Mnemonics, self.Services, self.elm, self.calc )
        if dr.type=='Parameter':
          datastr, help, csvd = get_parameter( self.Parameters[dr.name], self.Mnemonics, self.Services, self.elm, self.calc )
        if dr.type=='Identification':
          datastr, help, csvd = get_identification( self.Identifications[dr.name], self.Mnemonics, self.Services, self.elm, self.calc )       
        if dr.type=='Command':
          datastr = dr.name + " [Command] " + self.Commands[dr.name].label
          
        if mod_globals.opt_csv and csvf!=0 and (dr.type=='State' or dr.type=='Parameter'):
          csvline = csvline + ";" + str(csvd)
                 
        strlst.append( datastr )
        if mod_globals.opt_verbose and len(help)>0:
          tmp_str = ''
          for s in help:
            s = s.replace('\n',' ')
            s = s.replace('\r',' ')
            tmp_str = tmp_str + s + ' '
          i = 0
          W = 50
          while i*W<len(tmp_str):
            strlst.append('\t'+tmp_str[i*W:(i+1)*W])
            i=i+1
          strlst.append('')

      clearScreen()
      header =  "ECU : "+self.ecudata['ecuname']+'  '+self.ecudata['doc']+'\n'
      header = header + "Screen : "+path
      print pyren_encode( header )
      
      H = 25
      pages = len(strlst)/H
      for l in strlst[page*H:(page+1)*H]:
        print pyren_encode( l )

      if pages>0:
        print ""
        print "[Page ",page+1," from ",pages+1,"] Press page number to switch or any other to exit"
      else:  
        print ""
        print "Press any key to exit"
      
      if kb.kbhit():
        c = kb.getch()
        if len(c)!=1: continue
        n = ord(c)-ord('0')
        if n>0 and n<=(pages+1):
          page = n-1
          continue
        kb.set_normal_term()
        if mod_globals.opt_csv and csvf!=0:
          csvf.close()
        return
       
      # check refresh rate 
      tc = time.time()
      if ((tc-tb)<self.minimumrefreshrate):
        time.sleep(tb+self.minimumrefreshrate-tc)
      tb = tc
      
  def show_subfunction(self, subfunction, path):
    while(1): 
      clearScreen()
      header = "ECU : "+self.ecudata['ecuname']+'  '+self.ecudata['doc']+'\n'
      header = header + "Screen : "+path+" -> "+subfunction.text 
      print pyren_encode( header )
         
      menu = []
      if len(subfunction.datarefs)!=0 and len(subfunction.datarefs)>0:
        self.show_datarefs(subfunction.datarefs, path+" -> "+subfunction.text )
        return
      else:
        return        
   
  def show_function(self, function, path):
    while(1): 
      clearScreen()
      header =  "ECU : "+self.ecudata['ecuname']+'  '+self.ecudata['doc']+'\n'
      header = header + "Screen : "+path+" -> "+function.text
      print pyren_encode( header ) 
         
      menu = []
      if len(function.datarefs)!=0 and len(function.subfunctions)!=0:
        print "Warning: both datarefs and functions not empty"
      if len(function.subfunctions)!=0:
        for sfu in function.subfunctions:
          menu.append(sfu.text)
        menu.append("<Up>")
        choice = Choice(menu, "Choose :")
        if choice[0]=="<Up>": return
        self.show_subfunction(function.subfunctions[int(choice[1])-1], path+" -> "+function.text )    
      if len(function.datarefs)!=0:
        self.show_datarefs(function.datarefs, path+" -> "+function.text )
        return        
   
  def show_screen(self, screen):
    while(1): 
      clearScreen()
      header = "ECU : "+self.ecudata['ecuname']+'  '+self.ecudata['doc']+'\n'
      header = header + "Screen : "+screen.name    
      print pyren_encode( header ) 
      
      menu = []
      if len(screen.datarefs)!=0 and len(screen.functions)!=0:
        print "Warning: both datarefs and functions not empty"
      if len(screen.functions)!=0:
        for fu in screen.functions:
          menu.append(fu.text)
        menu.append("<Up>")
        choice = Choice(menu, "Choose :")
        if choice[0]=="<Up>": return
        self.show_function(screen.functions[int(choice[1])-1],screen.name)    
      if len(screen.datarefs)!=0:
        self.show_datarefs(screen.datarefs, screen.name)
        return        
  
  def show_defaults_std_a(self):
    while(1):
      path = "DE (STD_A)"
      clearScreen()
      header =  "ECU : "+self.ecudata['ecuname']+'  '+self.ecudata['doc']+'\n'
      header = header + "Screen : " + path 
      print pyren_encode( header ) 
          
      menu = []
      
      defstr = {}
      hlpstr = {}
  
      self.elm.clear_cache()
  
      dtcs, defstr, hlpstr = get_default_std_a( self.Defaults, self.Mnemonics, self.Services, self.elm, self.calc, self.getDTCmnemo )
      
      listkeys = defstr.keys()
      for d in listkeys:
        menu.append(defstr[d])

      menu.append("<Up>")
      menu.append("<Clear>")
      choice = Choice(menu, "Choose one for detailed view or <Clear>:")
  
      if choice[0]=="<Up>": return
      if choice[0]=="<Clear>": 
        print "Executing command ", self.resetDTCcommand
        executeCommand( self.Commands[self.resetDTCcommand], self, self.elm, header )
        return
      
      index = int(choice[1])-1
      dtchex = listkeys[index] if len(listkeys) > index else listkeys[0]
      
      path = path+' -> '+defstr[dtchex]+'\n\n'+hlpstr[dtchex]+'\n'
      
      self.show_datarefs(self.Defaults[dtchex[:4]].datarefs, path) 
    
  def show_defaults_std_b(self):
    while(1):
      path = "DE (STD_B)"
      clearScreen()
      header = "ECU : "+self.ecudata['ecuname']+'  '+self.ecudata['doc']+'\n'
      header = header + "Screen : " + path
      print pyren_encode( header ) 
           
      menu = []
      
      defstr = {}
      hlpstr = {}
  
      self.elm.clear_cache()
  
      dtcs, defstr, hlpstr = get_default_std_b( self.Defaults, self.Mnemonics, self.Services, self.elm, self.calc, self.getDTCmnemo )
      
      listkeys = defstr.keys()
      for d in listkeys:
        menu.append(defstr[d])
         
      menu.append("<Up>")
      menu.append("<Clear>")
      choice = Choice(menu, "Choose one for detailed view or <Clear>:")
  
      if choice[0]=="<Up>": return
      if choice[0]=="<Clear>": 
        print "Executing command ", self.resetDTCcommand
        executeCommand( self.Commands[self.resetDTCcommand], self, self.elm, header )
        return
          
      index = int(choice[1])-1
      dtchex = listkeys[index] if len(listkeys) > index else listkeys[0]

      path = path+' -> '+defstr[dtchex]+'\n\n'+hlpstr[dtchex]+'\n'
      
      self.show_datarefs(self.Defaults[dtchex[:4]].datarefs, path) 
    
  def show_defaults_failflag(self):
    while(1):
      path = "DE (FAILFLAG)"
      clearScreen()
      header =  "ECU : "+self.ecudata['ecuname']+'  '+self.ecudata['doc']+'\n'      
      header = header + "Screen : " + path
      print pyren_encode( header ) 
           
      menu = []
      
      defstr = {}
      hlpstr = {}
  
      self.elm.clear_cache()
      
      dtcs, defstr, hlpstr = get_default_failflag( self.Defaults, self.Mnemonics, self.Services, self.elm, self.calc )
      
      for d in sorted(defstr.keys()):
        menu.append(defstr[d])
         
      menu.append("<Up>")
      menu.append("<Clear>")
      choice = Choice(menu, "Choose one for detailed view or <Clear>:")
  
      if choice[0]=="<Up>": return
      if choice[0]=="<Clear>": 
        print "Executing command ", self.resetDTCcommand
        #header = "ECU : "+self.ecudata['ecuname']+'  '+self.ecudata['doc']+'\n'+ \
        #     "Screen : "+path
        executeCommand( self.Commands[self.resetDTCcommand], self, self.elm, header )
        return
          
      dtchex = dtcs[int(choice[1])-1]
      
      path = path+' -> '+defstr[dtchex]+'\n\n'+hlpstr[dtchex]+'\n'
      
      self.show_datarefs(self.Defaults[dtchex].datarefs, path) 
    
  def show_screens(self):
    while(1):
      clearScreen()
      header = "ECU : "+self.ecudata['ecuname']+'  '+self.ecudata['doc']+'\n'
      print pyren_encode( header ) 

      menu = []
      for l in self.screens:
        if l.name=="DE": l.name = "DE : Device errors"
        if l.name=="ID": l.name = "ID : Identifications"
        if l.name=="SY": l.name = "SY : System state"
        if l.name=="LC": l.name = "LC : System configuration"
        if l.name=="SP": l.name = "SP : System parameters"
        if l.name=="AC": l.name = "AC : Ececuting tests"
        if l.name=="CF": l.name = "CF : Changing configuration"
        if l.name=="VP": l.name = "VP : VIN programming"
        if l.name=="RZ": l.name = "RZ : Resets"
        if l.name=="SC": l.name = "SC : Configuration scenarios"
        if l.name=="SCS": l.name = "SCS : Security configuration scenarios"
        if l.name=="EZ": l.name = "EZ : EZSTEP"
        if l.name=="ED": l.name = "ED : DE extra information"
        menu.append(l.name)
      if mod_globals.opt_cmd : menu.append("ECM : Extended command set")
      menu.append("<Up>")
      choice = Choice(menu, "Choose :")
      if choice[0]=="<Up>": return
      
      if choice[0][:2]=="DE": 
        if self.ecudata['stdType']=='STD_A':
          self.show_defaults_std_a()
        if self.ecudata['stdType']=='STD_B' or self.ecudata['stdType']=='UDS' :
          self.show_defaults_std_b()
        if self.ecudata['stdType']=='FAILFLAG':
          self.show_defaults_failflag()
        continue

      if choice[0][:3]=="ECM":  
        scrn = ecu_screen( "ECM" ) 
        scrn.datarefs = []
        for cm in sorted(self.Commands):
          scrn.datarefs.append( ecu_screen_dataref("",cm,"Command")) 
        self.show_screen(scrn)
        continue
          
      self.show_screen(self.screens[int(choice[1])-1])

def bukva( bt, l ):
  S1 = chr((bt-l) % 26 + ord('A'))
  ex = int(bt-l) / 26
  if ex:
    S2 = chr((ex-1) % 26 + ord('A'))
    S1 = S2 + S1
  return S1

def gen_equ( m ):

  l = len(m.request)/2+1  
  sb = int(m.startByte)
  bl = int(m.bitsLength)
  
  equ = bukva(sb,l)
  
  if bl>8  and bl<=16:
    equ = bukva(sb,l)+'*256+'+bukva(sb+1,l)
  if bl>16 and bl<=24:
    equ = bukva(sb,l)+'*65536+'+bukva(sb+1,l)+'*256+'+bukva(sb+2,l)
  if bl>24 and bl<=32:
    equ = bukva(sb,l)+'*16777216+'+bukva(sb+1,l)+'*65536+'+bukva(sb+2,l)+'*256+'+bukva(sb+3,l)
    
  if m.littleEndian == '1':
    if bl>8  and bl<=16:
      equ = bukva(sb+1,l)+'*256+'+bukva(sb,l)
    if bl>16 and bl<=24:
      equ = bukva(sb+2,l)+'*65536+'+bukva(sb+1,l)+'*256+'+bukva(sb,l)
    if bl>24 and bl<=32:
      equ = bukva(sb+3,l)+'*16777216+'+bukva(sb+2,l)+'*65536+'+bukva(sb+1,l)+'*256+'+bukva(sb,l)

  if len(equ)>2:
    if equ[0]=='(' and equ[-1]==')':
      equ = equ
    else:
      equ = '('+equ+')'
  
  exbits = bl%8
  if exbits:
    rshift = 8-int(m.startBit)-exbits
    andmask = str(2**bl-1)
    if bl==1:
      equ = "{"+equ+":"+str(rshift)+"}"
      #equ = "{"+equ+":"+str(m.startBit)+"}"    
    elif rshift==0:
      equ = "("+equ+"&"+andmask+")"
    else:
      #equ = "(("+equ+">"+rshift+")&"+andmask+")"
      equ = "(("+equ+">"+str(rshift)+")&"+andmask+")"
  
  if m.type.startswith('SNUM'):
    if equ[0]=='(' and equ[-1]==')':
      equ = 'signed'+equ
    else:
      equ = 'signed('+equ+')'

  return equ 
  
if __name__ == "__main__":
  
  os.chdir(os.path.dirname(os.path.realpath(sys.argv[0])))

  if len(sys.argv)<3:
    print "Usage: mod_ecu.py <Family+ID> <language> [torq] [nochk]"
    print "Example:"
    print "   mod_ecu.py 0110016 RU "
    sys.exit(0)
  
  
  Defaults        = {}
  Parameters      = {}
  States          = {}
  Identifications = {}
  Commands        = {}
  Services        = {}
  Mnemonics       = {}
  
  print "Loading language "
  sys.stdout.flush()                                         
  lang = optfile("../Location/DiagOnCan_"+sys.argv[2]+".bqm",True)
  print "Done"
  sys.stdout.flush()                                         
  
  fgfile = "../EcuRenault/Sessions/FG"+sys.argv[1]+".xml"
  sgfile = "../EcuRenault/Sessions/SG"+sys.argv[1]+".xml"

  mdom = xml.dom.minidom.parse(fgfile)
  mdoc = mdom.documentElement
  
  print "Loading optimyzer"
  sys.stdout.flush()                                         
  opt_file = optfile(sgfile)
  
  print "Loading defaults"
  df_class = ecu_defaults       ( Defaults,        mdoc, opt_file.dict, lang.dict )
  print "Loading parameters"
  pr_class = ecu_parameters     ( Parameters,      mdoc, opt_file.dict, lang.dict )
  print "Loading states"
  st_class = ecu_states         ( States,          mdoc, opt_file.dict, lang.dict )
  print "Loading identifications"
  id_class = ecu_identifications( Identifications, mdoc, opt_file.dict, lang.dict )
  print "Loading commands"
  cm_class = ecu_commands       ( Commands,        mdoc, opt_file.dict, lang.dict )
  print "Loading mnemonics"
  mm_class = ecu_mnemonics      ( Mnemonics,       mdoc, opt_file.dict, lang.dict )
  
  if len(sys.argv)==3:
    print 
    print "Defaults"
    print 
    for i in sorted(Defaults.keys()):
      print Defaults[i].name,"[",i,"] ",Defaults[i].label
  
    print 
    print "Parameters"
    print 
    for i in sorted(Parameters.keys()):
      print Parameters[i].codeMR,"[",i,"] ",Parameters[i].label
  
    print 
    print "States"
    print 
    for i in sorted(States.keys()):
      print States[i].codeMR,"[",i,"] ",States[i].label
  
    print 
    print "Identifications"
    print 
    for i in sorted(Identifications.keys()):
      print Identifications[i].codeMR,"[",i,"] ",Identifications[i].label
  
    print 
    print "Commands"
    print 
    for i in sorted(Commands.keys()):
      print Commands[i].codeMR,"[",i,"] ",Commands[i].label
      
    sys.exit(0)
    
  if len(sys.argv)>3 and sys.argv[3].upper()!='TORQ':
    sys.exit(0)
    
  family = sys.argv[1][:2]
  eindex = sys.argv[1][2:]  
  sss = snat[F2A[family]]
  ddd = dnat[F2A[family]]
  filename  = "PR_"+ddd+"_"+sss+"_"+eindex+"_"+sys.argv[2]+".csv"
  cf = open( filename, "w")
     
  line = "%s,%s,%s,%s,%s,%s,%s,%s\n"%("name","ShortName","ModeAndPID","Equation","Min Value","Max Value","Units","Header")
  cf.write( line )
  
  memIt = []
  for i in sorted(Parameters.keys()):
    
    if Parameters[i].codeMR in memIt:
      continue
    else:
      memIt.append(Parameters[i].codeMR)
      
    m = Parameters[i].mnemolist[0]
    
    if len(Parameters[i].mnemolist)!=1:
      continue
    if '?' in Parameters[i].computation: 
      if len(sys.argv)==5 and sys.argv[4].upper()=='NOCHK':
        pass
      else:
        continue
    if Mnemonics[m].bitsLength=='':
      continue
    if Mnemonics[m].startBit=='':
      continue
    if Mnemonics[m].startByte=='':
      continue
      
    equ = gen_equ( Mnemonics[m] )   

    c_name = Parameters[i].label.replace(',','.')
    c_short = Parameters[i].codeMR
    c_pid = Mnemonics[m].request
    c_equ = Parameters[i].computation.replace( m, equ )
    c_min = Parameters[i].min
    c_max = Parameters[i].max
    c_unit = Parameters[i].unit
    
    line = '"[PR]%s","%s","%s","%s","%s","%s","%s","%s"\n'%(c_name,c_short,c_pid,c_equ,c_min,c_max,c_unit,ddd)
    cf.write( line.encode('utf-8') )

  memIt = []
  for i in sorted(States.keys()):

    if States[i].codeMR in memIt:
      continue
    else:
      memIt.append(States[i].codeMR)
      
    m = States[i].mnemolist[0]
    
    if len(States[i].mnemolist)!=1:
      continue
    if Mnemonics[m].bitsLength=='':
      continue
    if Mnemonics[m].startBit=='':
      continue
    if Mnemonics[m].startByte=='':
      continue
    
    equ = gen_equ( Mnemonics[m] )   

    c_name = States[i].label.replace(',','.')
    c_short = States[i].codeMR
    c_pid = Mnemonics[m].request
    if len(sys.argv)==5 and sys.argv[4].upper()=='NOCHK':
      c_equ = States[i].computation.replace( m, equ )
    else:
      c_equ = equ
    c_min = '0'
    c_max = '0'
    c_unit = ''
    
    line = '"[ST]%s","%s","%s","%s","%s","%s","%s","%s"\n'%(c_name,c_short,c_pid,c_equ,c_min,c_max,c_unit,ddd)
    cf.write( line.encode('utf-8') )

  cf.close()
  
  print
  print "File:", filename, "created"
  print 
  print "Init string for CAN:"
  print "ATAL\\nATSH"+ddd+"\\nATCRA"+sss+"\\nATFCSH"+ddd+"\\nATFCSD300000\\nATFCSM1\\nATSP6\\n10C0"
  #print "ATSH"+ddd+"\\nATCRA"+sss+"\\nATSP6\\n10C0"
  print
  print 
  print "Init string for Engine K-line (FAST INIT):"
  print "ATSH81"+F2A[family]+"F1\\nATSW96\\nATIB10\\nATSP5\\nATFI\\n10C0"
  print
  print "Init string for Engine K-line (SLOW INIT):"
  print "ATSH81"+F2A[family]+"F1\\nATSW96\\nATIB10\\nATSP4\\nATSI\\n10C0"
  print
  
