#!/usr/bin/python

import struct
import sys
import os
import pickle
import time

from xml.dom.minidom import parseString

class optfile:

  dict = {}
  
  def __init__(self,filename,progress=False,cache=True):
    
    self.dict = {}

    fn = filename
    pn = './cache/'+os.path.basename(fn)+'.p' 
    
    if os.path.isfile(pn):
      self.dict = pickle.load( open( pn, "rb" ) )        
    elif os.path.isfile(fn):
      lf = open( fn, "rb" )
      self.get_dict( lf, progress )
      if cache:
        pickle.dump( self.dict, open( pn, "wb" ) )
    
  def get_2_bytes(self,lf):
    c = [0x5,0x4,0x7,0x6,0x1,0x0,0x3,0x2,0xd,0xc,0xf,0xe,0x9,0x8,0xb,0xa]

    i = lf.tell()
    bytes = lf.read(2)
    
    x = 0
    x = struct.unpack('<H', bytes)[0]
    if self.fb==0: return x
    
    y = 0
    y = c[i & 0xf]+(c[(i>>4) & 0xf]<<4)+(c[(i>>8) & 0xf]<<8)+(c[(i>>12) & 0xf]<<12)
    return x^y

  def get_4_bytes(self,lf):
    return self.get_2_bytes(lf)+(self.get_2_bytes(lf)<<16)

  def get_dict(self,lf,progress):
    
    self.fb = ord(lf.read(1))
    if self.fb!=0x55: return
    
    lf.seek(0x14)
    protlen = self.get_4_bytes(lf)
    lf.seek(0x18+protlen*2)
    keyoff = self.get_4_bytes(lf)
      
    lf.seek(keyoff-8)
    tb = self.get_4_bytes(lf)
    i = keyoff
    
    n = 0
    
    while i<tb:
    
      if progress and (i&0xff==0):
        pr = (i+2-keyoff)*100/(tb-keyoff)
        print '\r['+'X'*(pr/2)+' '*(50-pr/2)+'] '+str(int(pr))+'%',
        sys.stdout.flush()
          
      lf.seek(i)
      addr = self.get_4_bytes(lf)
      strl = self.get_4_bytes(lf)
      keyl = self.get_4_bytes(lf)
      n = n + 1
      
      j = 0
      key = ''
      while j<keyl:
        bytes = self.get_2_bytes(lf)
        key += unichr(bytes)
        j=j+1
      
      lf.seek(addr)
      j = 0
      line = ''
      while j<strl:
        bytes = self.get_2_bytes(lf)
        line += unichr(bytes)
        j=j+1
      line = line.strip()
      line = line.strip('\n')
      line = line.replace(u'\xab','\"')
      line = line.replace(u'\xbb','\"')

      self.dict[key] = line 
      i=i+12+keyl*2
    
    if progress:
      print '\r['+'X'*50+'] 100%',

'''
  def get_dict(self,lf,progress):
    self.fb = ord(lf.read(1))
    if self.fb!=0x55: return
    lf.seek(0x2022)
    tb = self.get_4_bytes(lf)
    i = 0x202a
    while i<tb:
    
      if progress and (i&0xff==0):
        pr = (i-0x2029)*100/(tb-0x202a)
        print '\r['+'X'*(pr/2)+' '*(50-pr/2)+'] '+str(int(pr))+'%',
        sys.stdout.flush()
          
      lf.seek(i)
      addr = self.get_4_bytes(lf)
      strl = self.get_4_bytes(lf)
      keyl = self.get_4_bytes(lf)
      j = 0
      key = ''
      while j<keyl:
        bytes = self.get_2_bytes(lf)
        key += unichr(bytes)
        j=j+1
      
      lf.seek(addr)
      j = 0
      line = ''
      while j<strl:
        bytes = self.get_2_bytes(lf)
        line += unichr(bytes)
        j=j+1
      line = line.strip()
      line = line.strip('\n')
      line = line.replace(u'\xab','\"')
      line = line.replace(u'\xbb','\"')

      self.dict[key] = line 
      i=i+12+keyl*2
    
    if progress:
      print '\r['+'X'*50+'] 100%',
    
'''
if __name__ == "__main__":

  os.chdir(os.path.dirname(os.path.realpath(sys.argv[0])))
  
  if len(sys.argv)==1:
    print "Usage: mod_optfile.py <filename> [key]"
    print "       mod_optfile.py ALLSG"
    print "Example:"
    print "   mod_optfile.py ../Location/DiagOnCan_RU.bqm"
    print "   mod_optfile.py ../EcuRenault/Sessions/SG0110016.XML P001"
    sys.exit(0)

  if sys.argv[1]=='ALLSG':
    for subdir, dirs, files in os.walk('../EcuRenault/Sessions'):
      for file in files:
        if file.startswith('SG'):
          sgFileName = '../EcuRenault/Sessions/S'+file[1:]
          ugFileName = '../EcuRenault/Sessions/U'+file[1:]
          #print sgFileName
          of = optfile(sgFileName,False,False)
          rf = '<?xml version="1.0" ?><DiagOnCan>'
          for k in sorted(of.dict.keys()):
            if of.dict[k][:1]=='<' and of.dict[k][-1:]=='>':
              rf = rf + of.dict[k]
          rf = rf + '</DiagOnCan>'
          rf = parseString(rf).toprettyxml(indent='  ')
          print ugFileName
          f = open(ugFileName,'wt')
          f.write( rf )
          f.close()
    exit(0) 
  
  of = optfile(sys.argv[1])
  
  if len(sys.argv)==2:
    for k in sorted(of.dict.keys()):
      print '#'*60
      print 'Key:',k
      print '-'*60
      if of.dict[k][:1]=='<' and of.dict[k][-1:]=='>':
        print parseString(of.dict[k]).toprettyxml(indent='  ')     
      else:
        print of.dict[k]
   
  if len(sys.argv)==3:
    k = sys.argv[2]
    if k in of.dict.keys():
      if of.dict[k][:1]=='<' and of.dict[k][-1:]=='>':
        print parseString(of.dict[k]).toprettyxml(indent='  ')     
      else:
       print of.dict[k]
    else:
      for i in sorted(of.dict.keys()):
        if k in i:
          print '#'*60
          print 'Key:',i
          print '-'*60
          if of.dict[i][:1]=='<' and of.dict[i][-1:]=='>':
            print parseString(of.dict[i]).toprettyxml(indent='  ')    
          else:
            print of.dict[i]
      
  
   
