#!/usr/bin/python

import sys, os
import mod_globals

os.chdir(os.path.dirname(os.path.realpath(sys.argv[0])))

import pickle

mod_globals.os = os.name
 
if mod_globals.os == 'nt':
  import pip
  
  try:
    import psutil
  except ImportError:
    pip.main(['install','psutil'])  
  try:
    import psutil
    p = psutil.Process(os.getpid())
    p.nice(psutil.HIGH_PRIORITY_CLASS)
  except ImportError:
    pass

  try:
    import serial
  except ImportError:
    pip.main(['install','pyserial'])

  try:
    import colorama
  except ImportError:
    pip.main(['install','colorama'])
    try:
      import colorama
    except ImportError:
      print "\n\n\n\t\t\tGive me access to the Internet for download modules\n\n\n"
      sys.exit()
  colorama.init()
else:
  # let's try android
  try:
    import androidhelper as android
    mod_globals.os = 'android'
  except:
    try:
      import android
      mod_globals.os = 'android'
    except:
      pass
  
if mod_globals.os != 'android':    
  try:
    import serial
    from serial.tools  import list_ports
    #import ply
  except ImportError:
    print "\n\n\n\tPleas install additional modules"
    print "\t\t>sudo easy_install pyserial"
    #print "\t\t>sudo easy_install ply"
    sys.exit()
    
from mod_elm       import ELM
from mod_scan_ecus import ScanEcus
from mod_ecu       import ECU
from mod_optfile   import *
from mod_utils     import *


def optParser():
  '''Parsing of command line parameters. User should define at least com port name'''
  
  import optparse

  parser = optparse.OptionParser(
    usage = "%prog -p <port> [options]",
    version="%prog 0.9.9(beta)",
    description = "pyRen - python version of Reanult car diagnostic"
  )
  
  parser.add_option("-p",
      help="ELM327 com port name",
      dest="port",
      default="",
      type="string")

  parser.add_option("-s",
      help="com port speed configured on ELM {38400[default],57600,115200,230400,500000} DEPRECATED",
      dest="speed",
      default="38400",
      type="string")

  parser.add_option("-r",
      help="com port rate during diagnostic session {38400[default],57600,115200,230400,500000}",
      dest="rate",
      default="38400",
      type="string")

  parser.add_option("-L",
      help="language option {RU[default],GB,FR,IT,...}",
      dest="lang",
      default="RU")

  parser.add_option("-m",
      help="number of car model",
      dest="car",
      default=0,
      type="int")

  parser.add_option("-v",
      help="show parameter explanations",
      dest="verbose",
      default=False,
      action="store_true")

  parser.add_option("-e",
      help="index of ECU (like as 10016)",
      dest="ecuid",
      default="",
      type="string")

  parser.add_option("--log",
      help="log file name",
      dest="logfile",
      default="",
      type="string")

  parser.add_option("--demo",
      help="for debuging purpose. Work without car and ELM",
      dest="demo",
      default=False,
      action="store_true")

  parser.add_option("--csv",
      help="save data in csv format",
      dest="csv",
      default=False,
      action="store_true")

  parser.add_option("--scan",
      help="scan ECUs even if savedEcus.p file exists",
      dest="scan",
      default=False,
      action="store_true")


  (options, args) = parser.parse_args()
  
  if not options.port and mod_globals != 'android':
    parser.print_help()
    iterator = sorted(list(list_ports.comports()))
    print ""
    print "Available COM ports:"
    for port, desc, hwid in iterator:
      print "%-30s \n\tdesc: %s \n\thwid: %s" % (port,desc,hwid)
    print ""
    exit(2)
  else:
    mod_globals.opt_port    = options.port
    mod_globals.opt_ecuid   = options.ecuid
    mod_globals.opt_speed   = int(options.speed)
    mod_globals.opt_rate    = int(options.rate)
    mod_globals.opt_lang    = options.lang
    mod_globals.opt_car     = options.car
    mod_globals.opt_log     = options.logfile
    mod_globals.opt_demo    = options.demo
    mod_globals.opt_scan    = options.scan
    mod_globals.opt_csv     = options.csv
    mod_globals.opt_verbose = options.verbose

def main():
  '''Main function'''
  
  optParser()
  
  print 'Opening ELM'
  elm = ELM( mod_globals.opt_port, mod_globals.opt_speed, mod_globals.opt_log )

  #change serial port baud rate 
  if mod_globals.opt_speed<mod_globals.opt_rate and not mod_globals.opt_demo:
    elm.port.soft_boudrate( mod_globals.opt_rate )
     
  print 'Loading ECUs list'
  se  = ScanEcus(elm)                    #Prepare list of all ecus

  if not os.path.isfile("savedEcus.p") or mod_globals.opt_scan: 
    # choosing model 
    se.chooseModel( mod_globals.opt_car )  #choose model of car for doing full scan
    se.scanAllEcus()                       #First scan of all ecus
  
  print "Loading language "
  sys.stdout.flush()
                                         #loading language data
  lang = optfile("../Location/DiagOnCan_"+mod_globals.opt_lang+".bqm",True)
  mod_globals.language_dict = lang.dict
  print "Done"
  
  #check cache version
  if not os.path.isfile("./cache/version099.txt"):
    #if the cache has old version then we should clear it
    for root, dirs, files in os.walk("./cache"):
      for sfile in files:
        if sfile.startswith("ver") or sfile.startswith("FG"):
          full_path = os.path.join("./cache", sfile)
          os.remove(full_path)
    #create new version file
    verfile = open( "./cache/version099.txt", "wb" )
    verfile.write("Do not remove me if you have v.0.9.9 or above.\n")
    verfile.close()

  while( 1 ):
    clearScreen()
    choosen_ecu = se.chooseECU( mod_globals.opt_ecuid )   # choose ECU among detected
    mod_globals.opt_ecuid = ''
    if choosen_ecu==-1:
      continue
    
    ecucashfile = "./cache/"+choosen_ecu['ModelId']+".p"
    
    if os.path.isfile(ecucashfile):                         #if cache exists
      ecu = pickle.load( open( ecucashfile, "rb" ) )        #load it
    else:                                                   #else
      ecu = ECU(choosen_ecu, lang.dict )                    #loading original data for chosen ECU
      pickle.dump( ecu, open( ecucashfile, "wb" ) )         #and save cache
    
    ecu.initELM( elm )                                      #init ELM for chosen ECU 
    
    ecu.show_screens()                                      # show ECU screens

if __name__ == '__main__':  
  main()
  
